# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-14 07:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bee_django_mission', '0007_auto_20180613_1555'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomUserMission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('custom_name', models.CharField(blank=True, max_length=180, null=True, verbose_name='\u81ea\u5b9a\u4e49\u540d\u5b57')),
                ('custom_count', models.IntegerField(blank=True, null=True, verbose_name='\u81ea\u5b9a\u4e49\u6570\u91cf')),
                ('mission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.Mission')),
                ('stage', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.Stage')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['pk'],
                'db_table': 'bee_django_mission_custom_user_mission',
                'verbose_name': '\u5b66\u751f\u81ea\u5b9a\u4e49\u4efb\u52a1',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-05-25 08:00
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app_label', models.CharField(max_length=180, verbose_name='app\u540d')),
                ('model', models.CharField(max_length=180, verbose_name='\u6a21\u5757\u540d')),
                ('user_field', models.CharField(max_length=180, verbose_name='\u7528\u6237\u5b57\u6bb5\u540d')),
                ('info', models.CharField(max_length=180, null=True, verbose_name='\u5907\u6ce8')),
            ],
            options={
                'ordering': ['id'],
                'db_table': 'bee_django_mission_content_type',
            },
        ),
        migrations.CreateModel(
            name='Line',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='\u6807\u9898')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('line_type', models.IntegerField(choices=[(1, '\u957f\u671f\u4efb\u52a1'), (2, '\u5468\u4efb\u52a1')], default=0)),
                ('auto_finish', models.BooleanField(default=True, verbose_name='\u662f\u5426\u81ea\u52a8\u5b8c\u6210')),
                ('auto_start', models.BooleanField(default=True, verbose_name='\u662f\u5426\u81ea\u52a8\u5f00\u542f\u4e0b\u4e00\u4e2a')),
            ],
            options={
                'ordering': ['id'],
                'db_table': 'bee_django_mission_line',
                'verbose_name': '\u4efb\u52a1\u7ebf',
            },
        ),
        migrations.CreateModel(
            name='Mission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='\u6807\u9898')),
                ('count', models.IntegerField(verbose_name='\u6570\u91cf')),
                ('info', models.TextField(blank=True, null=True, verbose_name='\u5907\u6ce8')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('order_by', models.IntegerField(blank=True, null=True, verbose_name='\u987a\u5e8f')),
            ],
            options={
                'ordering': ['created_at'],
                'db_table': 'bee_django_mission',
                'verbose_name': '\u4efb\u52a1',
            },
        ),
        migrations.CreateModel(
            name='MissionType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, unique=True, verbose_name='\u6807\u9898')),
                ('aggregate_type', models.IntegerField(choices=[(1, 'Count'), (2, 'Sum'), (3, 'TruncDay')], default=1, verbose_name='\u805a\u5408\u7c7b\u578b')),
                ('field_name', models.CharField(default='id', max_length=180, verbose_name='\u53d6\u503c\u5b57\u6bb5\u540d')),
                ('timestamp_field', models.CharField(blank=True, max_length=180, null=True, verbose_name='\u65f6\u95f4\u5b57\u6bb5\u540d')),
                ('comparison_type', models.IntegerField(choices=[(1, '>='), (2, '>')], default=1, verbose_name='\u6bd4\u8f83\u7c7b\u578b')),
                ('operator_type', models.IntegerField(choices=[(0, '\u65e0'), (1, '* 60')], default=0, verbose_name='\u5bf9\u503c\u8fd0\u7b97')),
                ('conditions', models.TextField(blank=True, help_text='\u683c\u5f0f\u4e3a\uff1a[\u6761\u4ef61\uff1a\u503c1\uff0c\u6761\u4ef62\uff1a\u503c2]\uff0c\u591a\u4e2a\u6761\u4ef6\u7528,\u5206\u5272', null=True, verbose_name='\u5176\u4ed6\u9644\u52a0\u6761\u4ef6')),
                ('link_url', models.CharField(blank=True, max_length=180, null=True, verbose_name='\u94fe\u63a5\u5730\u5740')),
                ('link_name', models.CharField(blank=True, max_length=180, null=True, verbose_name='\u94fe\u63a5\u540d\u5b57')),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_mission.ContentType', verbose_name='app\u53camodel')),
            ],
            options={
                'ordering': ['id'],
                'db_table': 'bee_django_mission_type',
                'verbose_name': '\u4efb\u52a1\u7c7b\u578b',
            },
        ),
        migrations.CreateModel(
            name='Stage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('level', models.IntegerField(null=True, verbose_name='\u9636\u6bb5')),
                ('name', models.CharField(max_length=180, verbose_name='\u6807\u9898')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('line', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_mission.Line')),
            ],
            options={
                'ordering': ['level'],
                'db_table': 'bee_django_mission_stage',
                'verbose_name': '\u9636\u6bb5\u4efb\u52a1',
            },
        ),
        migrations.CreateModel(
            name='UserLine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_at', models.DateTimeField(auto_now_add=True)),
                ('finish_at', models.DateTimeField(null=True)),
                ('line', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.Line')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['start_at'],
                'db_table': 'bee_django_mission_user_line',
                'verbose_name': '\u5b66\u751f\u4efb\u52a1\u7ebf',
            },
        ),
        migrations.CreateModel(
            name='UserMission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('custom_name', models.CharField(blank=True, max_length=180, null=True, verbose_name='\u81ea\u5b9a\u4e49\u540d\u5b57')),
                ('custom_count', models.IntegerField(blank=True, null=True, verbose_name='\u81ea\u5b9a\u4e49\u6570\u91cf')),
                ('finish_at', models.DateTimeField(null=True)),
                ('mission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.Mission')),
            ],
            options={
                'ordering': ['finish_at'],
                'db_table': 'bee_django_mission_user_mission',
                'verbose_name': '\u5b66\u751f\u7684\u4efb\u52a1',
            },
        ),
        migrations.CreateModel(
            name='UserStage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, null=True, verbose_name='\u6807\u9898')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='\u6dfb\u52a0\u65f6\u95f4')),
                ('start_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='\u5f00\u59cb\u65f6\u95f4')),
                ('finish_at', models.DateTimeField(null=True, verbose_name='\u5b8c\u6210\u65f6\u95f4')),
                ('end_at', models.DateTimeField(null=True, verbose_name='\u7ed3\u675f\u65f6\u95f4')),
                ('stage', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.Stage')),
                ('user_line', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.UserLine')),
            ],
            options={
                'ordering': ['start_at'],
                'db_table': 'bee_django_mission_user_stage',
                'verbose_name': '\u5b66\u751f\u9636\u6bb5\u4efb\u52a1',
            },
        ),
        migrations.AddField(
            model_name='usermission',
            name='user_stage',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bee_django_mission.UserStage'),
        ),
        migrations.AddField(
            model_name='mission',
            name='mission_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_mission.MissionType'),
        ),
        migrations.AddField(
            model_name='mission',
            name='stage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_mission.Stage'),
        ),
        migrations.AlterUniqueTogether(
            name='contenttype',
            unique_together=set([('app_label', 'model')]),
        ),
        migrations.AlterUniqueTogether(
            name='stage',
            unique_together=set([('line', 'level')]),
        ),
    ]



# -*- coding:utf-8 -*-
__author__ = 'bee'

from django import forms
from django.core.exceptions import NON_FIELD_ERRORS

from .models import Mission, Stage, UserLine, Line, UserMission
from .utils import check_add_line_stage_to_user


class StageForm(forms.ModelForm):
    class Meta:
        model = Stage
        fields = ['line', "name", 'level', 'prize_min', 'prize_max']
        unique_together = ("line", "level")
        error_messages = {
            NON_FIELD_ERRORS: {
                'unique_together': "阶段不能重复",
            }
        }


class StageUnlimitedForm(StageForm):
    l = Line.objects.filter(line_type=1)
    line = forms.ModelChoiceField(queryset=l, label='所属任务线', required=True)



    # def validate_unique(self):
    #     exclude = self._get_validation_exclusions()
    #     # exclude.remove('level') # allow checking against the missing attribute
    #
    #     try:
    #         self.instance.validate_unique(exclude=exclude)
    #     except forms.ValidationError, e:
    #         self._update_errors(e.message_dict)

    # def clean(self):
    #     level = self.cleaned_data['level']
    #     print(self)
    #     stage_list = Stage.objects.filter(level=level, line__line_type=1)
    #     if stage_list.exists():
    #         raise forms.ValidationError(u"阶段不能重复")
    #     return self.cleaned_data


class MissionForm(forms.ModelForm):
    class Meta:
        model = Mission
        fields = ['mission_type', "stage", "name", "count", "info", 'order_by']

    def __init__(self, line, *args, **kwargs):
        super(MissionForm, self).__init__(*args, **kwargs)
        stage_queryset = Stage.objects.filter(line=line)
        self.fields["stage"] = forms.ModelChoiceField(queryset=stage_queryset, label='所属阶段', required=True)


class UserLineCreateForm(forms.ModelForm):
    class Meta:
        model = UserLine
        fields = ['line']

    def __init__(self, *args, **kwargs):
        super(UserLineCreateForm, self).__init__(*args, **kwargs)
        line_queryset = Line.objects.filter(line_type=1)
        self.fields["line"] = forms.ModelChoiceField(queryset=line_queryset, label='任务线', required=True)

    def clean(self):
        line = self.cleaned_data['line']
        res, msg = check_add_line_stage_to_user(line)
        if not res:
            raise forms.ValidationError(msg)
        return self.cleaned_data


class UserMissionForm(forms.ModelForm):
    class Meta:
        model = UserMission
        fields = ['custom_name', 'custom_count']

    def __init__(self, *args, **kwargs):
        super(UserMissionForm, self).__init__(*args, **kwargs)
        if not self.initial["custom_name"]:
            self.initial["custom_name"] = self.instance.mission.name
        if not self.initial["custom_count"]:
            self.initial["custom_count"] = self.instance.mission.count



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from django.conf.urls import include, url
from . import views

app_name = 'bee_django_mission'
urlpatterns = [
    url(r'^test$', views.test, name='test'),
    url(r'^$', views.MissionList.as_view(), name='index'),
    # ======stage========
    url(r'^stage/unlimited/list/$', views.StageUnlimitedList.as_view(), name='stage_unlimited_list'),
    # url(r'^detail/(?P<pk>[0-9]+)/$', views.StageDetail.as_view(), name='mission_detail'),
    url(r'^stage/add/(?P<line_id>[0-9]+)/$', views.StageCreate.as_view(), name='stage_add'),
    url(r'^stage/update/(?P<pk>[0-9]+)/$', views.StageUpdate.as_view(), name='stage_update'),
    # url(r'^delete/(?P<pk>[0-9]+)/$', views.MissionDelete.as_view(), name='mission_delete'),

    # ===== mission======
    url(r'^list/$', views.MissionList.as_view(), name='mission_list'),
    url(r'^detail/(?P<pk>[0-9]+)/$', views.MissionDetail.as_view(), name='mission_detail'),
    url(r'^add/(?P<line_type>[0-9]+)/$', views.MissionCreate.as_view(), name='mission_add'),
    url(r'^update/(?P<line_type>[0-9]+)/(?P<pk>[0-9]+)/$', views.MissionUpdate.as_view(), name='mission_update'),
    url(r'^delete/(?P<pk>[0-9]+)/$', views.MissionDelete.as_view(), name='mission_delete'),




    # ===user line ====
    url(r'^user/line/create/(?P<user_id>[0-9]+)/$', views.UserLineCreate.as_view(),
        name='user_line_add'),

    # ======user stage =====
    url(r'^user/stage/detail/(?P<pk>[0-9]+)/$', views.UserStageDetail.as_view(),
        name='user_stage_detail'),
    url(r'^user/stage/update/status/(?P<user_stage_id>[0-9]+)/(?P<status>[0-9]+)/$',
        views.UserStageUpdateStatus.as_view(), name='user_stage_update_status'),
    url(r'^user/stage/update/mission/(?P<user_stage_id>[0-9]+)/$', views.UserStageUpdateMission.as_view(),
        name='user_stage_update_mission'),


    # ====user mission =====
    # 获取学生的周任务
    url(r'^user/mission/list/week/(?P<user_id>[0-9]+)/$', views.UserMissionListWeek.as_view(), name='user_line_week'),
    # 获取学生的长线任务
    url(r'^user/mission/list/unlimited/(?P<user_id>[0-9]+)/$', views.UserMissionListUnlimited.as_view(),
        name='user_line_unlimited'),
    url(r'^user/mission/update/(?P<pk>[0-9]+)/$', views.UserStageUpdate.as_view(), name='user_stage_update'),

    # ==== user stage finish count ======
    url(r'^user/stage/finish/rank/list/(?P<line_id>[0-9]+)/$', views.UserStageFinishRankList.as_view(), name='user_stage_finish_rank_list'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-14 09:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_mission', '0009_auto_20180614_1705'),
    ]

    operations = [
        migrations.AddField(
            model_name='userstage',
            name='prize_coin',
            field=models.IntegerField(default=0, verbose_name='\u5956\u52b1\u7684m\u5e01'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-13 07:55
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_mission', '0006_userstagetfinishcount'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='UserStagetFinishCount',
            new_name='UserStageFinishCount',
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

import pytz, calendar, time
from datetime import datetime, tzinfo, timedelta

LOCAL_TIMEZONE = pytz.timezone('Asia/Shanghai')


# 获取现在时间，默认返回北京时间
def get_now(tz=LOCAL_TIMEZONE):
    now = datetime.now(tz)
    return now


# 获取今日0点时间，默认返回北京时间
def get_today(tz=LOCAL_TIMEZONE):
    now = get_now()
    today_str = now.strftime("%Y-%m-%d")
    today = datetime.strptime(today_str, "%Y-%m-%d")
    today = tz.localize(today)  # 加时区
    # print(now, today)
    return today


# 获取当周周日晚23点23分23秒
def get_current_week_range_datetime():
    today = get_today()
    start_date = today - timedelta(days=today.weekday())
    end_date = start_date + timedelta(days=7)
    end_date = end_date - timedelta(seconds=1)
    # print(start_date,end_date, '1')
    return start_date, end_date
